#!/usr/bin/python
# -*- coding: UTF-8 -*-


from jxORM.orm.jxUtils import checkAssert, Now, transValue2DB, transValue2Datetime, judgeTimeIsCurrent, getRenameTableName, getDateTime_to
#from jxORM.orm.common import DB, DBType, #get_db_type
from jxORM.orm.common import DB, DBType
from jxORM.orm.orm import ORM

# 支持分表【不支持按周】的、for循环查表【单表】
# 使用方法：
# s = query(dbname)
#
# 设置分表信息，如果其中有的表是分表的话
# s.setRename
#
# 添加要查询的表
# s.addTable(table1,fieldPrefix='user')
# s.addTable(table2,fieldPrefix='demo')
#
# 添加查询条件
# s.addTableJoin
# s.Equal
# s.GreatEqual
# 如果使用到了分表，则一定要有其分表字段上的GreatEqual、Great
# 否则无法判断要从哪开始查询
#
# 准备查询
# s.start()
#
# 开始查询
# for d in s:
#
class query:
    def __init__(self, dbname, limit=15):
        self._dbname = dbname
        self._dbtype = None #get_db_type(dbname)
        self._tables = {}
        # 当前表查询结束
        self._queryCurrentTable = False

        self._limit = limit
        self._offset = 0
        self._data = []
        # 比较的条件
        self._conditionCompare = None
        self._conditionCompare_join = None
        self._sql = None
        self._sqlTemple = None
        self._sql_exec = None

        # 是否需要分表
        self._renameType = None
        self._currentTable = None
        # 需要分表的表名
        self._renameTalbeName = None
        # 需要分表的表别名
        self._renameTalbeAlias = None
        # 判断分表的时间字段
        self._renameJudgeByField = None
        # 分表时，查询顺序是按时间的升序、还是降序，默认升序
        self._renameDec = False
        self._renameMin = None
        self._renameMax = None
        # 所有涉及到的分表
        self._renameTalbe = []

        self._orderBy = ''

    def set_limit(self, limit):
        self._limit = limit

    def set_sql(self, sql):
        self._sql_exec = sql

    def setRename(self, renameTalbeName, renameType='day', renameJudgeByField='CreateTime', renameDec=False):
        self._renameType = renameType
        self._renameTalbeName = renameTalbeName
        self._renameJudgeByField = renameJudgeByField
        self._renameDec = renameDec

    # fieldPrefix:本表各列查出的数据添加什么样的前缀，fieldPrefix='user'，ID则为userID
    # wantSelectFields:希望查询哪些列，没有则全部查询
    def addTable(self, clsName, fieldPrefix=None, wantSelectFields=[]):
        tc = ORM.getCls(clsName)
        checkAssert(not tc is None, '需searchBy查询的数据类【{}】应先注册', clsName)
        t = {}
        t['clsName'] = clsName
        t['cls'] = tc
        if fieldPrefix is None:
            fieldPrefix = f't{len(self._tables)}'
        t['fieldPrefix'] = fieldPrefix
        al = tc.__dict__['_ormAttr']
        fl = []
        if not wantSelectFields is None:
            if len(wantSelectFields) == 0:
                for attr in al:
                    fa = {'col': attr, 'alias': f'{fieldPrefix}{attr}'}
                    fl.append(fa)
            else:
                for attr in wantSelectFields:
                    checkAssert(attr in al, '条件键名【{}】不在数据类【{}】属性列表中', attr, clsName)
                    fa = {'col': attr, 'alias': f'{fieldPrefix}{attr}'}
                    fl.append(fa)

        t['fields'] = fl
        self._tables[fieldPrefix] = t

        if not self._renameTalbeName is None and self._renameTalbeName == clsName:
            self._renameTalbeAlias = fieldPrefix

        return fieldPrefix

    def set_order_by(self, tableAlias1, field, desc=False):
        self._orderBy = f' ORDER BY {tableAlias1}.{field} '
        if desc:
            self._orderBy += 'DESC '

    def _getSelect_count(self):
        return 'Select Count(1) totalCount '

    def _getSelect(self):
        ss = None
        for ta in self._tables:
            t = self._tables[ta]
            fl = t['fields']
            for f in fl:
                if ss is None:
                    ss = f'{ta}.{f["col"]} AS {f["alias"]}'
                else:
                    ss = ss + f', {ta}.{f["col"]} AS {f["alias"]}'
        return ss

    def _getFrom(self):
        sf = None
        for ta in self._tables:
            if not self._renameTalbeAlias is None and self._renameTalbeAlias == ta:
                tn = '--renameTalbeName--'
            else:
                t = self._tables[ta]
                tn = t['clsName']
            if sf is None:
                sf = f'{tn} AS {ta}'
            else:
                sf = sf + f', {tn} AS {ta}'
        return sf

    # 多表时两个表的连接条件
    def addTableJoin(self, tableAlias1, field1, tableAlias2, field2):
        co1 = self._tables.get(tableAlias1, None)
        checkAssert(not co1 is None, '表别名尚未定义：{}', tableAlias1)
        fty1 = co1['cls'].__dict__['_ormAttr'].get(field1)
        cn1 = co1['clsName']
        checkAssert(not fty1 is None, '【{}】不在数据类【{}】属性列表中', field1, cn1)
        co2 = self._tables.get(tableAlias2, None)
        checkAssert(not co2 is None, '表别名尚未定义：{}', tableAlias2)
        fty2 = co2['cls'].__dict__['_ormAttr'].get(field2)
        cn2 = co2['clsName']
        checkAssert(not fty2 is None, '【{}】不在数据类【{}】属性列表中', field2, cn2)
        checkAssert(fty1 == fty2, '【{}.{}】的类型【{}】和【{}.{}】的类型【{}】不同',
                            cn1, field1, fty1, cn2, field2, fty2)

        if self._conditionCompare_join is None:
            self._conditionCompare_join = f'{tableAlias1}.{field1}={tableAlias2}.{field2}'
        else:
            self._conditionCompare_join = self._conditionCompare_join + f' AND {tableAlias1}.{field1}={tableAlias2}.{field2}'

        self._conditionCompare = self._conditionCompare_join

        return self

    def ClearCondition(self):
        self._conditionCompare = self._conditionCompare_join

    def Equal(self, clsAlias, field, value):
        self._addCompare(clsAlias, field, '=', value)
        return self

    def NoEqual(self, clsAlias, field, value):
        self._addCompare(clsAlias, field, '<>', value)
        return self

    def GreatEqual(self, clsAlias, field, value):
        self._addCompare(clsAlias, field, '>=', value)
        return self

    def Great(self, clsAlias, field, value):
        self._addCompare(clsAlias, field, '>', value)
        return self

    def LessEqual(self, clsAlias, field, value):
        self._addCompare(clsAlias, field, '<=', value)
        return self

    def Less(self, clsAlias, field, value):
        self._addCompare(clsAlias, field, '<', value)
        return self

    # 全文匹配
    def Match(self, clsAlias, field, value):
        self._addCompare(clsAlias, field, 'match', value)
        return self

    def _addCompare(self, clsAlias, field, op, value):
        co = self._tables.get(clsAlias, None)
        checkAssert(not co is None, '表别名尚未定义：{}', clsAlias)
        tan = co.get('claName', None)
        tc = co.get('cls', None)
        al = tc.__dict__['_ormAttr']
        ft = al.get(field, None)
        checkAssert(not ft is None, '条件属性【{}】不在数据类【{}】属性列表中', field, tan)

        if ft == 'datetime':
            value = transValue2Datetime(value)
            if field == self._renameJudgeByField:
                if op == '<' or op == '<=':
                    self._renameMax = value
                elif op == '>' or op == '>=':
                    self._renameMin = value
                else:
                    raise Exception('时间字段的比较应该是：<、<=、>、>=')

        fvs = transValue2DB(value)

        if op == 'match':
            ct = f'MATCH ({clsAlias}.{field}) AGAINST ("{fvs}")'
        else:
            ct = f'{clsAlias}.{field}{op}{fvs}'

        if self._conditionCompare is None:
            self._conditionCompare = ct
        else:
            self._conditionCompare = f'{self._conditionCompare} AND {ct}'

    def start(self):
        self._offset = 0
        self._sql = None
        self._sqlTemple = None
        self._queryCurrentTable = True
        if self._renameTalbeName is None:
            return
        checkAssert(not self._renameMin is None, '【{}】分表，必须指定查询开始时间', self._renameTalbeName)
        current = Now()
        if self._renameMax is None:
            max = current
        else:
            max = self._renameMax

        dt = self._renameMin
        while dt < max:
            b = judgeTimeIsCurrent(current, dt, renameType=self._renameType)
            if b:
                tn = self._renameTalbeName
            else:
                tn = getRenameTableName(self._renameTalbeName, dt, renameType=self._renameType)
            self._renameTalbe.append(tn)
            dt = getDateTime_to(dt, renameType=self._renameType)

        if self._renameDec:
            # 要求降序
            self._renameTalbe.reverse()

    def _doSearch(self):
        while True:
            if not self._renameTalbeAlias is None:
                if self._currentTable is None:
                    if len(self._renameTalbe) == 0:
                        return False
                    self._currentTable = self._renameTalbe.pop(0)
                    self._queryCurrentTable = True
                    # 开始新表
                    self._offset = 0
                    self._sql = None
            b, rc = self._search(self._dbname)
            if b:
                return True
            # 当前表已经查完了，这时有两种可能：self._data中有数据或没有
            if not self._renameTalbeAlias is None:
                if len(self._renameTalbe) == 0:
                    self._currentTable = None
                else:
                    self._currentTable = self._renameTalbe.pop(0)
                self._queryCurrentTable = True
                # 开始新表
                self._offset = 0
                self._sql = None
            if len(self._data) > 0:
                # 有数据
                return True
            if self._renameTalbeAlias is None:
                # 单表就直接结束，带分表则还需要检查是否还有其它分表
                return False

    def totalCount(self):
        total = 0
        self.start()
        if len(self._renameTalbe) > 0:
            for tn in self._renameTalbe:
                n = self._getCount(self._dbname, tn)
                total = total + n[0]
        else:
            total = self._getCount(self._dbname, None)[0]
        return total

    @DB
    def _getCount(self, db, tn):
        sf = self._getFrom()
        if self._conditionCompare is None:
            st = f'Select Count(1) totalCount FROM {sf}'
        else:
            st = f'Select Count(1) totalCount FROM {sf} WHERE {self._conditionCompare}'
        if not self._renameTalbeAlias is None:
            sql = st.replace('--renameTalbeName--', tn)
        else:
            sql = st

        try:
            db.cursor.execute(sql)
        except Exception as e:
            errMsg = str(e)
            if 'Table ' in errMsg and " doesn't exist" in errMsg:
                return 0
        rs = db.cursor.fetchone()
        return rs.get('totalCount', 0)

    def list(self, offset=None, remove_prefix=False):
        self._data = []
        if offset is not None:
            self._offset = offset
        self._list(self._dbname)
        if remove_prefix:
            rs = []
            for rd in self._data:
                d = {}
                for k, v in rd:
                    ks = k.split('__')
                    if len(ks) == 2:
                        k = ks[1]
                    d[k] = v
                rs.append(d)
            self._data = rs
        return self._data

    def get(self, remove_prefix=False):
        self._data = []
        self._offset = 0
        rs = self.list(self._dbname, remove_prefix=remove_prefix)
        if len(rs) == 0:
            return None
        return rs[0]

    def list2(self, offset=None):
        self._data = []
        if offset is not None:
            self._offset = offset
        self._list(self._dbname)
        return self._data

    def count(self):
        return self._getCount(self._dbname, None)[0]

    @DB
    def _list(self, db):
        if self._sql is None:
            if self._sql_exec is not None:
                self._sql = f'{self._sql_exec} {self._orderBy}LIMIT {self._limit}'
            else:
                if self._sqlTemple is None:
                    ss = self._getSelect()
                    sf = self._getFrom()
                    if self._conditionCompare is None:
                        self._sqlTemple = f'SELECT {ss} FROM {sf} {self._orderBy}LIMIT {self._limit}'
                    else:
                        self._sqlTemple = f'SELECT {ss} FROM {sf} WHERE {self._conditionCompare} {self._orderBy}LIMIT {self._limit}'
                if not self._renameTalbeAlias is None:
                    self._sql = self._sqlTemple.replace('--renameTalbeName--', self._currentTable)
                else:
                    self._sql = self._sqlTemple

        sql = self._sql + f' OFFSET {self._offset}'

        try:
            db.cursor.execute(sql)
            self._data = db.cursor.fetchall()
        except Exception as e:
            errMsg = str(e)
            if 'Table ' in errMsg and " doesn't exist" in errMsg:
                self._queryCurrentTable = False
                self._currentTable = None
                self._offset = 0
                self._sql = None
                return False
            raise e

    @DB
    def _search(self, db):
        if not self._queryCurrentTable:
            # 当前表已经失效了
            return False
        if self._sql is None:
            if self._sql_exec is not None:
                self._sql = f'{self._sql_exec} {self._orderBy}LIMIT {self._limit}'
            else:
                if self._sqlTemple is None:
                    ss = self._getSelect()
                    sf = self._getFrom()
                    if self._conditionCompare is None:
                        self._sqlTemple = f'SELECT {ss} FROM {sf} {self._orderBy}LIMIT {self._limit}'
                    else:
                        self._sqlTemple = f'SELECT {ss} FROM {sf} WHERE {self._conditionCompare} {self._orderBy}LIMIT {self._limit}'
                if not self._renameTalbeAlias is None:
                    self._sql = self._sqlTemple.replace('--renameTalbeName--', self._currentTable)
                else:
                    self._sql = self._sqlTemple

        sql = self._sql + f' OFFSET {self._offset}'

        try:
            db.cursor.execute(sql)
        except Exception as e:
            errMsg = str(e)
            if 'Table ' in errMsg and " doesn't exist" in errMsg:
                self._queryCurrentTable = False
                self._currentTable = None
                self._offset = 0
                self._sql = None
                return False
            raise e

        self._data = db.cursor.fetchall()
        ds = len(self._data)
        if ds < self._limit:
            # 本表结束
            self._queryCurrentTable = False
            self._currentTable = None
        else:
            self._offset = self._offset + self._limit
        return self._queryCurrentTable

    def _get(self):
        if len(self._data) == 0:
            b = self._doSearch()
            if not b:
                return None
        if len(self._data) == 0:
            return None
        return self._data.pop(0)

    def __next__(self):
        rs = self._get()
        if rs is None:
            self._sql = None
            self._sqlTemple = None
            self._currentTable = None
            self._data = []
            self._renameTalbe = []
            self._offset = 0
            raise StopIteration("遍历完了")
        return rs

    def __iter__(self):
        return self



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import traceback
import datetime
import enum
import re
from functools import wraps

from jxORM.orm.jxUtils import logger, checkAssert, extract_str_from_quotes, GetClsName, CID, Now, transValue2DB, transValue2Datetime, judgeTimeIsCurrent, getRenameTableName, getDateTime_to, stringAdd, defaultValue
from jxORM.orm.common import DB, DBType, DBDataType, transFromDB, transToDB, get_db_result, ColType


reg_expression = re.compile('\s*(\w+)\s+(==|>|<|<>|>=|<=)\s+(.+)')
def parse_expression(exp):
    ss = exp.split('&')
    rs = []
    for s in ss:
        m = reg_expression.match(s)
        if m:
            sv = m.group(3)
            ins = extract_str_from_quotes(sv, '"')
            if len(ins) == 0:
                ins = extract_str_from_quotes(sv, "'")
            if len(ins) > 0:
                sv = ''.join(ins)
            rs.append((m.group(1), m.group(2), sv))
    return rs
def trans_op(op):
    if op == '==':
        return '='
    return op

_orm_cls_list = {}
_orm_son_cls_list = {}
_orm_parent_cls_list = {}

def get_by_type_id(db, cls_name, id):
    c = _orm_cls_list.get(cls_name, None)
    if c is None:
        return None
    return c.GetByID(db, id)

def get_by_type_name(db, cls_name, name):
    c = _orm_cls_list.get(cls_name, None)
    if c is None:
        return None
    return c.GetByName(db, name)

def list_ancestors(my_name):
    c = _orm_parent_cls_list.get(my_name, None)
    d = []
    while c is not None:
        d.append(c)
        c = _orm_parent_cls_list.get(c.Name(), None)
    return d

def list_descendant(my_name):
    c = _orm_son_cls_list.get(my_name, None)
    d = []
    while c is not None:
        d.append(c)
        c = _orm_son_cls_list.get(c.Name(), None)
    return d
def get_parent_Name(cls):
    pc = cls.__bases__[0]
    pn = GetClsName(pc)
    if pn == 'object':
        return None
    return pn
def get_parent(cls):
    pn = get_parent_Name(cls)
    if pn is None:
        return None
    return _orm_cls_list.get(pn, None)

def get_index(type_hints, dict_list):
    rs = {}
    for k in type_hints:
        v = dict_list.get(k, None)
        if v is None:
            continue
        if isinstance(v, ColType):
            continue
        elif isinstance(v, int):
            l = rs.get(v, None)
            if l is None:
                l = []
                rs[v] = l
            l.append(k)
    return rs

def ORM(cls):
    '''
    @ORM
    class Test:
        #id是主键
        id:DBDataType.Int = ColType.PrimaryKey
        name:DBDataType.String
    '''
    clsName = cls.__name__
    table_alias = f't_{clsName}'
    parent = get_parent(cls)
    parent_list = list_ancestors(clsName)
    try:
        type_hints = cls.__annotations__
    except:
        import inspect
        type_hints = inspect.get_annotations(cls)
    keys = [k for k in type_hints if k in cls.__dict__ and cls.__dict__[k] == ColType.PrimaryKey]
    indexes = get_index(type_hints, cls.__dict__)
    @wraps(cls, updated=())
    class Wrapper(cls):  # 继承原类
        @classmethod
        def Alias(cls):
            return table_alias
        @classmethod
        def Name(cls):
            return clsName
        @classmethod
        def Parent(cls):
            return parent
        @classmethod
        def Fields(cls):
            return type_hints
        @classmethod
        def is_attr(cls, name):
            return name in type_hints
        @classmethod
        def is_key(cls, name):
            return name in keys

        @classmethod
        def get_type(cls, name):
            return type_hints.get(name, None)

        @classmethod
        def get_type_python(cls, name):
            dt = type_hints.get(name, None)
            if dt == DBDataType.Int:
                return int
            elif dt == DBDataType.Float:
                return float
            elif dt == DBDataType.Bool:
                return bool
            elif dt == DBDataType.String:
                return str
            elif dt == DBDataType.DataTime:
                return datetime.datetime
            elif dt == DBDataType.Json:
                return dict
            return None

        @classmethod
        def create(cls, db):
            try:
                db.create_table(clsName, type_hints, keys, indexes)
            except:
                pr = traceback.format_exc()
                logger.error(pr)

        @classmethod
        def get_cls_with_field(cls, scl, field):
            if cls.is_attr(field):
                return cls
            for sc in scl:
                if sc.is_attr(field):
                    return sc
            return None

        @classmethod
        def get_ty_with_field(cls, field):
            dt = cls.get_type(field)
            if dt is not None:
                return dt
            for sc in parent_list:
                dt = sc.get_type(field)
                if dt is not None:
                    return dt
            return None

        @classmethod
        def get_where(cls, expression_list):
            s = None
            for attr, op, v in expression_list:
                dt = type_hints.get(attr, None)
                if dt is None:
                    continue
                if isinstance(v, str):
                    if v[0] == '\'' or v[0] == '"':
                        v = v[1:-1]
                op = trans_op(op)
                if op == 'like':
                    v = f'{v}%'
                ov = transToDB(dt, v)
                s = stringAdd(s, '{}{}{}'.format(f'{table_alias}.{attr}', op, ov), ' AND ')
            return s
        @classmethod
        def get_from(cls):
            s = f'{clsName} as {table_alias}'
            return s
        @classmethod
        def get_select(cls):
            s = f'{table_alias}.*'
            return s

        @classmethod
        def searchBy_expression_list(cls, db, expression_list, offset=0, limit=100):
            fl = type_hints.copy()
            ss = cls.get_select()
            fs = cls.get_from()
            ws = cls.get_where(expression_list)
            for sc in parent_list:
                fl.update(sc.Fields())
                ss = stringAdd(ss, sc.get_select(), ',')
                fs = stringAdd(fs, sc.get_from(), ',')
                ws = stringAdd(ws, f'{table_alias}.ID={sc.Alias()}.ID', ' AND ')
                ws = stringAdd(ws, sc.get_where(expression_list), ' AND ')
            sql1 = f'SELECT {ss} FROM {fs} WHERE {ws}'
            if limit > 0:
                sql = sql1 +f' LIMIT {limit} OFFSET {offset}'
            else:
                sql = sql1
            db.execute(sql)
            rl = db.fetchall()
            return get_db_result(rl, db.description(), [cls], need_trans=db.need_trans())

        @classmethod
        def searchBy(cls, db, expression, offset:int=0, limit:int=100):
            '''
            以等式搜索数据
            '''
            expression_list = parse_expression(expression)
            return cls.searchBy_expression_list(db, expression_list, offset=offset, limit=limit)

        @classmethod
        def getBy(cls, db, expression):
            expression_list = parse_expression(expression)
            return cls._getBy(db, expression_list)

        @classmethod
        def _getBy(cls, db, expression_list):
            rl = cls.searchBy_expression_list(db, expression_list, offset=0, limit=2)
            if len(rl) == 0:
                return None
            checkAssert(len(rl) == 1, f'{clsName}::_getBy[{expression_list}] 查询结果数量不为1')
            return rl[0]

        @classmethod
        def GetByID(cls, db, id):
            expression_list = [('ID', '=', id)]
            return cls._Get(db, expression_list)

        @classmethod
        def GetByName(cls, db, name):
            expression_list = [('Name', '=', name)]
            return cls._Get(db, expression_list)

        @classmethod
        def Get(cls, db, expression):
            expression_list = parse_expression(expression)
            return cls._Get(db, expression_list)

        @classmethod
        def _Get(cls, db, expression_list):
            data = cls._getBy(db, expression_list)
            if data is None:
                return None
            return Wrapper(**data)

        def __init__(self, **kwargs):
            self._data = {}
            self._changed = {}
            if len(kwargs) > 0:
                self.set(init=True, **kwargs)
            else:
                self._myInit()

        def _myInit(self):
            #自动设置ID
            self._set(init=True, ID=CID())
            #自动设置创建时间
            self._set(init=True, CreateTime=Now())
            #自动设置创建时间
            self._set(init=True, Timestamp=Now())

        def update(self, db):
            if len(self._changed) == 0:
                return
            Wrapper.my_update_transaction(db, self._data, self._changed)
            for sc in parent_list:
                sc.my_update_transaction(db, self._data, self._changed)
            self._changed = {}

        @classmethod
        def my_update_transaction(cls, db, data, changed):
            checkAssert(len(keys) > 0, '数据对象update必须设置key！！！')
            sql1 = 'UPDATE {} SET '.format(clsName)
            sql2 = ' WHERE '
            s1 = None
            s2 = None
            for attr in changed:
                dt = type_hints.get(attr, None)
                if dt is None:
                    continue
                v = data.get(attr, None)
                if v is None:
                    v = defaultValue(dt)
                s1 = stringAdd(s1, '{}={}'.format(attr, v), ',')
            for attr in keys:
                v = data.get(attr, None)
                checkAssert(not v is None, '键【{}】未设置值', attr)
                s2 = stringAdd(s2, '{}={}'.format(attr, v), ' AND ')
            sql = sql1 + s1 + sql2 + s2

            db.execute(sql)

        def insert(self, db):
            Wrapper.my_insert_transaction(db, self._data)
            for sc in parent_list:
                sc.my_insert_transaction(db, self._data)

        @classmethod
        def my_insert_transaction(cls, db, data):
            sql1 = 'INSERT INTO {}('.format(clsName)
            sql2 = ' VALUES('
            s1 = None
            s2 = None
            for attr, dt in type_hints.items():
                v = data.get(attr, None)
                if v is None:
                    v = transToDB(dt, defaultValue(dt))
                s1 = stringAdd(s1, attr, ',')
                s2 = stringAdd(s2, str(v), ',')
            sql1 = sql1 + s1 + ')'
            sql2 = sql2 + s2 + ')'
            sql = sql1 + sql2

            db.execute(sql)

        def data(self):
            rs = {}
            for key, v in self._data.items():
                c = Wrapper.get_cls_with_field(parent_list, key)
                if c is None:
                    continue
                dt = c.get_type(key)
                rs[key] = transFromDB(dt, v)
            return rs

        def _set(self, init=False, **kwargs):
            for key, value in kwargs.items():
                c = Wrapper.get_cls_with_field(parent_list, key)
                if c is None:
                    continue
                if not init:
                    checkAssert(not c.is_key(key), f'{clsName}::_set[{key}] 不支持设置主键')
                dt = c.get_type(key)
                if dt is not None:
                    self._data[key] = transToDB(dt, value)
                    if not init:
                        self._changed[key] = True

        def set(self, **kwargs):
            self._set(**kwargs)

        def get(self, name):
            c = Wrapper.get_cls_with_field(parent_list, name)
            if c is not None:
                d = super().__getattribute__('_data')
                dv = d.get(name)
                dt = c.get_type(name)
                return transFromDB(dt, dv)
            return None

        def __getattribute__(self, name):
            c = Wrapper.get_cls_with_field(parent_list, name)
            if c is not None:
                d = super().__getattribute__('_data')
                dv = d.get(name)
                dt = c.get_type(name)
                return transFromDB(dt, dv)
            f = super().__getattribute__(name)
            if f is not None:
                return f
            return None

        def __setattr__(self, name, value):
            c = Wrapper.get_cls_with_field(parent_list, name)
            if c is None:
                super().__setattr__(name, value)
                return
            checkAssert(not c.is_key(name), f'{clsName}::_setattr[{name}] 不支持设置主键')
            dt = c.get_type(name)
            self._data[name] = transToDB(dt, value)
            self._changed[name] = True

    _orm_cls_list[clsName] = Wrapper
    if parent is not None:
        _orm_son_cls_list[parent.Name()] = Wrapper
        _orm_parent_cls_list[clsName] = parent
    return Wrapper

reg_from = re.compile('\s*(\w+)(\s+[as|AS]\s+(\w+))?')
def extract_from_simple(sql):
    rs = []
    bsql = sql.upper()
    from_index = bsql.find("FROM")
    where_index = bsql.find("WHERE")
    fs = sql[from_index+5:where_index]
    ss = fs.split(',')
    for s in ss:
        s = s.strip()
        m = reg_expression.match(s)
        if m:
            rs.append(m.group(1))
    return rs

def select(db, sql:str, need_trans=True):
    bsql = sql.upper().strip()
    bss = bsql.split(' ')
    checkAssert(bss[0] == 'SELECT', 'select只支持SELECT语句')
    cls_list = []
    if need_trans:
        fl = extract_from_simple(sql)
        for f in fl:
            c = _orm_cls_list.get(f, None)
            if c is not None:
                cls_list.append(c)
    db.execute(sql)
    rl = db.fetchall()
    return get_db_result(rl, db.description(), cls_list, need_trans=need_trans and db.need_trans())






import os

os.makedirs("logs", exist_ok=True)

from .orm.jxUtils import set_host_id, logger as jxORMLogger
from .orm.orm import ORM, get_by_type_id, get_by_type_name, select
from .orm.common import DBDataType, ColType, jxDB, DBType, set_default_db, get_default_db, get_db, register_create_db

__all__ = ['set_host_id', 'jxORMLogger', 'ORM', 'select', 'DBDataType', 'ColType', 'jxDB', 'DBType', 'set_default_db', 'get_default_db', 'get_db', 'register_create_db', ]



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import traceback
import datetime
import json

from functools import wraps
from threading import RLock
import datetime
import enum

from jxORM.orm.jxUtils import logger, checkAssert, transValue2Datetime, transValue2Dict, transValue2String, stringAdd, trip_quotes

class DBType(enum.IntEnum):
    No = 0
    SQLite = 1
    MySQL = 2

    @classmethod
    def from_string(cls, s):
        if s == 'mysql':
            return DBType.MySQL
        if s == 'sqlite':
            return DBType.SQLite
        return DBType.No

class ColType(enum.IntEnum):
    PrimaryKey = 0
    Index = 1

class DBDataType(enum.IntEnum):
    No = -1
    Int = 1
    Long = 2
    Float = 3
    Double = 4
    Bool = 5
    Chars = 6
    String = 7
    DataTime = 8
    Json = 9

    @classmethod
    def from_string(cls, s):
        if s == 'int':
            return DBDataType.Int
        if s == 'long':
            return DBDataType.Long
        if s == 'float':
            return DBDataType.Float
        if s == 'double':
            return DBDataType.Double
        if s == 'bool':
            return DBDataType.Bool
        if s == 'chars':
            return DBDataType.Chars
        if s == 'string':
            return DBDataType.String
        if s == 'datetime':
            return DBDataType.DataTime
        if s == 'json':
            return DBDataType.Json
        return DBDataType.No


_dbConfig = {}
_db_create_funcs = {}
_dbPool = {}
_db_name = None
def set_default_db(dbName):
    global _db_name
    _db_name = dbName
def get_default_dbname():
    return _db_name
def register_create_db(db_type, func):
    _db_create_funcs[db_type] = func
def get_ty(cl, attr):
    for c in cl:
        dt = c.get_ty_with_field(attr)
        if dt is not None:
            return dt
    return None
def get_db_result(result_list, descriptions, cls_list:list, need_trans=True):
    if result_list is None:
        return None
    if not need_trans or descriptions is None:
        return result_list
    column_names = [description[0] for description in descriptions]
    rs = []
    for r in result_list:
        result = dict(zip(column_names, r))
        d = {}
        rs.append(d)
        for k, v in result.items():
            dt = get_ty(cls_list, k)
            if dt is not None:
                d[k] = transFromDB(dt, v)
                continue
            d[k] = v
    return rs

def transFromDB(dt: DBDataType, dv):
    if dt == DBDataType.Int:
        return int(dv)
    if dt == DBDataType.Long:
        return int(dv)
    if dt == DBDataType.Float:
        return float(dv)
    if dt == DBDataType.Double:
        return float(dv)
    if dt == DBDataType.Bool:
        try:
            return bool(dv)
        except:
            return False
    if dt == DBDataType.String:
        s = str(dv)
        return trip_quotes(s)
    if dt == DBDataType.Chars:
        s = str(dv)
        return trip_quotes(s)
    if dt == DBDataType.DataTime:
        if isinstance(dv, str):
            dv = trip_quotes(dv)
        return transValue2Datetime(dv)
    if dt == DBDataType.Json:
        if isinstance(dv, str) and (dv.startswith('"') or dv.startswith("'")):
            dv = dv[1:-1]
        return transValue2Dict(dv)
    return None

def transToDB(dt: DBDataType, dv):
    if dt == DBDataType.Int:
        return int(dv)
    if dt == DBDataType.Long:
        return int(dv)
    if dt == DBDataType.Float:
        return float(dv)
    if dt == DBDataType.Double:
        return float(dv)
    if dt == DBDataType.Bool:
        try:
            b = bool(dv)
            if b:
                return 1
            return 0
        except:
            return 0
    if dt == DBDataType.String:
        return f'\"{str(dv)}\"'
    if dt == DBDataType.Chars:
        return f'\"{str(dv)}\"'
    if dt == DBDataType.DataTime:
        if isinstance(dv, datetime.datetime):
            return f'\"{dv.strftime("%Y-%m-%d %H:%M:%S")}\"'
        elif isinstance(dv, str):
            return f'\"{dv}\"'
        else:
            try:
                dv = int(dv)
                dt = datetime.datetime.fromtimestamp(dv)
                return f'\"{dt.strftime("%Y-%m-%d %H:%M:%S")}\"'
            except:
                return ''
    if dt == DBDataType.Json:
        s = json.dumps(dv)
        return f"'{s}'"
    return None

class jxDB:
    @classmethod
    def set(cls, dbName, type=DBType.MySQL, is_default=True, host='127.0.0.1', port=3306, user='root', password='root', **kwargs):
        dc = jxDB(type=type, host=host, port=port, user=user, password=password)
        dc.attrs = kwargs
        _dbConfig[dbName] = dc
        if is_default:
            set_default_db(dbName)

    def __init__(self, type=DBType.MySQL, host='127.0.0.1', port=3306, user='root', password='root'):
        self.type = type
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.attrs = {}

class DB_interface:
    def __init__(self):
        self.conn = None
        self.cursor = None
        self._name = None

    def type(self):
        # 数据库类型
        return None

    def name(self):
        # 数据库类型
        return self._name

    def need_trans(self):
        return False

    def description(self):
        return None

    def _get_create_table_sql(self, table_name, fields, keys, indexs):
        return []
    def create_table(self, table_name, fields, keys, indexs):
        sl = self._get_create_table_sql(table_name, fields, keys, indexs)
        for sql in sl:
            self.execute(sql)

    def commit(self):
        self.conn.commit()

    def get_db_type(self, ty):
        return ''

    def rollback(self):
        self.conn.rollback()

    def execute(self, sql):
        logger.info(f'execute sql:{sql}')
        return self.cursor.execute(sql)

    def fetchone(self):
        return self.cursor.fetchone()

    def fetchall(self):
        return self.cursor.fetchall()

    def trans2String(self, vt, v):
        return transValue2String(vt, v, withQuote=True, to_db=True)

    def transaction(self):
        return DB_interface._transaction(self)

    def __enter__(self):
        if self.conn is not None:
            self.cursor = self.conn.cursor()
            self.conn.begin()
        return self

    def __exit__(self, type, value, trace):
        if self.conn is not None:
            self.cursor.close()
            self.conn.close()

    class _transaction:
        def __init__(self, db):
            self.db = db
        def __enter__(self):
            self.db.__enter__()
            return self
        def __exit__(self, type, value, trace):
            if type is None:
                self.db.commit()
            else:
                self.db.rollback()
            self.db.__exit__(type, value, trace)

class DB_mysql(DB_interface):
    def __init__(self, dbname):
        super().__init__()
        self._name = dbname
        self._pool = _dbPool.get(dbname)
        if self._pool is None:
            dc = _dbConfig.get(dbname)
            import pymysql
            from dbutils.pooled_db import PooledDB
            config = {
                'creator': pymysql,
                'host': dc.host,
                'port': dc.port,
                'user': dc.user,
                'password': dc.password,
                'db': dbname,
                'charset': "utf8",
                'cursorclass': pymysql.cursors.DictCursor
            }
            config.update(dc.attrs)
            self._pool = PooledDB(**config)
            _dbPool[dbname] = self._pool
        self.conn = self._pool.connection()

    def type(self):
        # 数据库类型
        return 'mysql'

    def get_db_type(self, ty):
        if ty == DBDataType.Int:
            return 'int'
        if ty == DBDataType.Long:
            return 'bigint'
        if ty == DBDataType.Float:
            return 'float'
        if ty == DBDataType.Double:
            return 'double'
        if ty == DBDataType.Bool:
            return 'tinyint'
        if ty == DBDataType.Chars:
            return 'varchar(126)'
        if ty == DBDataType.String:
            return 'mediumtext'
        if ty == DBDataType.DataTime:
            return 'datetime'
        if ty == DBDataType.Json:
            return 'mediumtext'
        return 'mediumtext'

    def _get_create_table_sql(self, table_name, fields, keys, indexs):
        rs = []
        fl = {}
        pk = None
        il = []
        for k, dt in fields.items():
            fs = f'`{k}` {self.get_db_type(dt)} NOT NULL'
            fl[k] = fs
        if len(keys) > 0:
            ks = None
            for k in keys:
                ks = stringAdd(ks, f'`{k}`', ',')
            pk = f'PRIMARY KEY ({ks})'
        if len(indexs) > 0:
            for i, ins in indexs.items():
                ifs = None
                for ikn in ins:
                    ifs = stringAdd(ifs, f'`{ikn}`', ',')
                tis = f'KEY `{table_name}_index_{i}` ({ifs})'
                il.append(tis)
        fs = ','.join(fl.values())
        fs = stringAdd(fs, pk, ',')
        ils = ','.join(il)
        fs = stringAdd(fs, ils, ',')
        cs = f'CREATE TABLE `{table_name}` ({fs}) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci;'
        rs.append(cs)
        return rs

class DB_sqlite(DB_interface):
    def __init__(self, dbname):
        super().__init__()
        self._name = dbname
        self._pool = _dbPool.get(dbname)
        if self._pool is None:
            import sqlite3
            from dbutils.pooled_db import PooledDB
            config = {
                'creator': sqlite3,
                'mincached': 5,
                'maxcached': 30,
                'check_same_thread': False,
                'database': f'{dbname}.db',
            }
            self._pool = PooledDB(**config)
            _dbPool[dbname] = self._pool
        self.conn = self._pool.connection()

    def type(self):
        # 数据库类型
        return 'sqlite'

    def need_trans(self):
        return True

    def description(self):
        return self.cursor.description

    def get_db_type(self, ty):
        if ty in [DBDataType.Int, DBDataType.Long, DBDataType.Float, DBDataType.Double,DBDataType.Bool]:
            return 'INTEGER'
        return 'TEXT'

    def _get_create_table_sql(self, table_name, fields, keys, indexs):
        rs = []
        fl = {}
        for k, dt in fields.items():
            fs = f'`{k}` {self.get_db_type(dt)} NOT NULL'
            fl[k] = fs
        for k in keys:
            fs = fl.get(k, None)
            fs += ' PRIMARY KEY'
            fl[k] = fs
        fls = ','.join(fl.values())
        cs = f'CREATE TABLE `{table_name}` ({fls});'
        rs.append(cs)

        for i, il in indexs.items():
            fs = None
            for ikn in il:
                fs = stringAdd(fs, f'`{ikn}`', ',')
            cis = f'CREATE INDEX {table_name}_index_{i} ON {table_name} ({fs});'
            rs.append(cis)
        return rs

def get_default_db():
    return get_db(_db_name)

def get_db(dbname=None):
    if dbname is None:
        dbname = _db_name
    if dbname is not None:
        dc = _dbConfig.get(dbname)
        if dc is not None:
            func = _db_create_funcs.get(dc.type, None)
            if func is not None:
                return func(dbname)
    return None

def DB(func):
    @wraps(func)
    def wrapper(obj, dbname, *args, **kw):
        db = get_db(dbname)
        checkAssert(not db is None, '数据库未配置:{}', dbname)
        with db:
            try:
                result = func(obj, db, *args, **kw)
                db.commit()
                return result
            except Exception as e:
                db.rollback()
                raise e
    return wrapper

register_create_db(DBType.MySQL, lambda dbname:DB_mysql(dbname))
register_create_db(DBType.SQLite, lambda dbname:DB_sqlite(dbname))



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import os
import threading
import types
import time
import random
import datetime
import json
import traceback
from threading import RLock
from decimal import *
from enum import Enum

import base64
import pytz

import logging
from logging.handlers import RotatingFileHandler

class ValueType(Enum):
    none = 'none'
    string = 'string'
    int = 'int'
    float = 'float'
    datetime = 'datetime'
    bool = 'bool'
    json = 'json'
    @staticmethod
    def from_string(s:str):
        try:
            return ValueType[s.lower()]
        except:
            return ValueType.none

'''
_func_get_user：
    传入参数：
        user：用户名
        pwd：密码
    返回值：
        用户
用户类要求具有如下的方法：
    #用户名
    def name(self):
        return self.name
    #用户名可能会重复，所以起一个本公司唯一的简称名
    def abbr(self):
        return self.abbr
    #用户所有的角色【包括一级角色，如技术部经理；二级角色，如经理】
    def roles(self):
        return self.roles
'''

class User:
    def __init__(self, name):
        self._name = name
        self._abbr = name
        self._roles = [ ]

    def name(self):
        return self._name
    def abbr(self):
        return self._abbr
    def roles(self):
        return self._roles

_func_get_user = None
def set_func_get_user(func):
    global _func_get_user
    _func_get_user = func
def get_user(user, pwd):
    if _func_get_user is not None:
        return _func_get_user(user, pwd)
    return User(user)


int_Max = 0x7ffffff
tz = pytz.timezone('Asia/Shanghai')

hostID = 1
def set_host_id(id):
    global hostID
    hostID = id

hostName = 'JingXi'
mask_hostID = (hostID & 0xFFFF) << 48
mask_ts = 0xFFFFFFFF
myID = 0

_LOG_FORMAT = "%(asctime)s [%(filename)s %(lineno)s] - %(levelname)s - %(message)s"
_LOG_maxBytes = 512 * 1024 * 1024
#rolling日志文件的备份数
_LOG_backupCount = 30

private_key_path = '../../secure/privateKey.pem'
_log_name = 'web.log'

_rsa_decryptor = None
def get_rsa_decryptor():
    global _rsa_decryptor
    if _rsa_decryptor is None:
        _rsa_decryptor = RSA_Decryptor(private_key_path)
    return _rsa_decryptor

_allObjCanClear = {}


def _asyncExec(func):
    try:
        func()
    except:
        pr = traceback.format_exc()
        logger.error(pr)
def asyncExec(func):
    threading.Thread(target=_asyncExec, args=[func]).start()

def list_methods(obj):
    return [m for m in dir(obj) if not m.startswith("__")]

def list_methods_cls(cls, mothod_type='instance'):
    import inspect
    all_methods = inspect.getmembers(cls, predicate=inspect.isfunction)
    if mothod_type is None:
        return all_methods
    elif mothod_type == 'instance':
        return {
            name: method
            for name, method in all_methods
            if isinstance(method, types.FunctionType)
            and not name.startswith("__")  # 排除特殊方法
        }
    elif mothod_type == 'classmethod':
        return {
            name: method
            for name, method in all_methods
            if isinstance(method, classmethod)
            and not name.startswith("__")  # 排除特殊方法
        }
    return {
        name: method
        for name, method in all_methods
        if isinstance(method, staticmethod)
        and not name.startswith("__")  # 排除特殊方法
    }
def trip_quotes(s):
    ins = extract_str_from_quotes(s, '"')
    if len(ins) == 0:
        ins = extract_str_from_quotes(s, "'")
    if len(ins) > 0:
        s = ''.join(ins)
    return s
def extract_str_from_quotes(s, quote):
    result = []
    inside = False

    for char in s:
        if char == quote and not inside:
            inside = True
        elif char == quote and inside:
            break
        elif inside:
            result.append(char)

    return result

def stringAdd(str, want, split=','):
    if str is None:
        return want
    if want is None:
        return str
    return str + split + want
def defaultValue(ty):
    from jxORM.orm.common import DBDataType
    if ty == 'int' or ty == DBDataType.Int:
        return 0
    if ty == 'long' or ty == DBDataType.Long:
        return 0
    if ty == 'float' or ty == DBDataType.Float:
        return 0.0
    if ty == 'double' or ty == DBDataType.Double:
        return 0.0
    if ty == 'chars' or ty == DBDataType.Chars:
        return ''
    if ty == 'string' or ty == DBDataType.String:
        return ''
    if ty == 'bool' or ty == DBDataType.Bool:
        return False
    if ty == 'decimal':
        return Decimal.from_float(0.0)
    if ty == 'datetime' or ty == DBDataType.DataTime:
        return '1970-01-01 00:00:00'
    if ty == 'json' or ty == DBDataType.Json:
        return {}
    return None
def base64_decode(msg):
    # 将Base64编码的字符串转换为字节形式
    # 将字符串转换为字节格式
    msg_bytes = msg.encode('utf-8')
    encoded_bytes = base64.b64decode(msg_bytes)
    # 将编码的字节形式转换为字符串并返回
    return encoded_bytes.decode('utf-8')

def base64_encode(msg):
    # 将字符串转换为字节格式
    msg_bytes = msg.encode('utf-8')
    # 执行Base64编码
    encoded_bytes = base64.b64encode(msg_bytes)
    # 将编码的字节形式转换为字符串并返回
    return encoded_bytes.decode('utf-8')

def GetClsName(cls):
    cn = cls.__name__
    cs = cn.split('.')
    return cs[-1]
def StringIsEmpty(str):
    if str is None:
        return True
    if str == '':
        return True
    return False
def CID():
    global myID
    if myID > int_Max:
        myID = 1
    else:
        myID = myID + 1
    t = time.time()
    ts = int(round(t * 1000))
    return mask_hostID | (((ts >> 10) & mask_ts) << 16) | (myID & 0x0000FFFF)
def Now():
    dt = datetime.datetime.now(tz)
    return dt
def checkAssert(b,msg,*vs):
    if not b:
        if len(vs) == 0:
            raise Exception(msg)
        else:
            raise Exception(msg.format(*vs))
def getLogger(loggerName):
    logger = logging.getLogger(loggerName)
    logger.propagate = False
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter(_LOG_FORMAT)
    fh = RotatingFileHandler(f"logs/{loggerName}", maxBytes=_LOG_maxBytes, backupCount=_LOG_backupCount)
    fh.setLevel(logging.INFO)
    fh.setFormatter(formatter)

    logger.addHandler(fh)
    return logger

def waitClear(name, objCanClear):
    logger.info(f'添加可清理对象：{name}')
    co = _allObjCanClear.get(name, None)
    checkAssert(co is None, f'已经添加过可清理对象【{name}')
    _allObjCanClear[name] = objCanClear
def getClearObj(name):
    return _allObjCanClear.get(name, None)
def clearObj(name):
    logger.info(f'清理对象：{name}')
    co = _allObjCanClear.pop(name, None)
    if co is not None:
        co.clear()
def transValue2Number(str):
    try:
        s = str.index('.')
        return float(str)
    except:
        return transValue2Int(str)
def transValue2Int(v):
    if v is None:
        return 0
    try:
        return int(v)
    except:
        return 0
#
#目前只用于sql条件添加，所以送入的值都应该是字符串类型
#
def transValue2String(vt, v, withQuote=False, to_db=False):
    if v is None:
        return None
    if vt == ValueType.bool:
        v = bool(v)
        if to_db:
            if v:
                return '1'
            return '0'
        return '{}'.format(v).lower()
    if vt == ValueType.int:
        v = int(v)
        return '{}'.format(v)
    if vt == ValueType.float:
        v = float(v)
        return '{}'.format(v)
    if vt == ValueType.string or vt == ValueType.datetime:
        v = str(v)
        if withQuote:
            return '\'{}\''.format(v)
        else:
            return v
    return ''
def transValue2DB(v):
    if v is None:
        return None
    if isinstance(v,bool):
        if v:
            return '1'
        return '0'
    if isinstance(v,int):
        return '{}'.format(v)
    if isinstance(v,float):
        return '{}'.format(v)
    if isinstance(v,dict):
        try:
            jvs = json.dumps(v)
        except:
            jvs = ''
        return '\'{}\''.format(jvs)
    if isinstance(v,str):
        return '\'{}\''.format(v)
    if isinstance(v,Decimal):
        return str(v)
    if isinstance(v,datetime.datetime):
        return '\'{}\''.format(v.timestamp())
    return None
#32位系统中最大的秒数，如果用datetime.max，由于太大，会报错：ValueError: year 10000 is out of range
_DataTime_max_second = 2147483647
def transValue2Datetime(v):
    if v is None:
        return None
    if isinstance(v,datetime.datetime):
        return v
    if isinstance(v, float) or isinstance(v, int):
        #v = int(v)
        if v > _DataTime_max_second:
            v = v / 1000
        dt = datetime.datetime.fromtimestamp(v).replace(tzinfo=tz)
        return dt
    if isinstance(v,str):
        dt = datetime.datetime.strptime(v, "%Y-%m-%d %H:%M:%S")
        return dt.astimezone(tz)
    return None
def transValue2Dict(v):
    if v is None:
        return {}
    if isinstance(v,dict):
        return v
    if isinstance(v,str):
        try:
            return json.loads(v)
        except:
            return {}
    return {}
def judgeTimeIsCurrent(dtCurrent, dt, renameType='day'):
    if renameType == 'day':
        return dtCurrent.year == dt.year and dtCurrent.month == dt.month and dtCurrent.day == dt.day
    elif renameType == 'month':
        dtfrom = transValue2Datetime(f'{dtCurrent.year}-{dtCurrent.month}-1 00:00:00')
        if dtCurrent.month == 12:
            dtto = transValue2Datetime(f'{dtCurrent.year+1}-1-1 00:00:00')
        else:
            dtto = transValue2Datetime(f'{dtCurrent.year}-{dtCurrent.month+1}-1 00:00:00')
    else:
        dtfrom = transValue2Datetime(f'{dtCurrent.year}-1-1 00:00:00')
        dtto = transValue2Datetime(f'{dtCurrent.year+1}-1-1 00:00:00')
    return dtfrom<=dt and dt<dtto
def getRenameTableName(tableName, dt, renameType='day'):
    if renameType == 'day':
        return f'{tableName}_{dt.year}_{dt.month}_{dt.day}'
    if renameType == 'month':
        return f'{tableName}_{dt.year}_{dt.month}'
    if renameType == 'year':
        return f'{tableName}_{dt.year}'
    return None
def getDateTime_from(dt, renameType='day'):
    if renameType == 'day':
        dtfrom = transValue2Datetime(f'{dt.year}-{dt.month}-{dt.day} 00:00:00')
    elif renameType == 'month':
        dtfrom = transValue2Datetime(f'{dt.year}-{dt.month}-{1} 00:00:00')
    else:
        dtfrom = transValue2Datetime(f'{dt.year}-{1}-{1} 00:00:00')
    return dtfrom
def getDateTime_to(dt, renameType='day'):
    if renameType == 'day':
        one = datetime.timedelta(days=1)
        dtfrom = getDateTime_from(dt,renameType='day')
        dtto = dtfrom + one
    elif renameType == 'month':
        if dt.month == 12:
            dtto = transValue2Datetime(f'{dt.year+1}-1-1 00:00:00')
        else:
            dtto = transValue2Datetime(f'{dt.year}-{dt.month+1}-1 00:00:00')
    else:
        dtto = transValue2Datetime(f'{dt.year+1}-1-1 00:00:00')
    return dtto

logger = getLogger('jxORM.log')



from setuptools import setup, find_packages
 
setup(
    name='jxORM',  # 你的库名
    version='0.2.0',    # 版本号
    packages=find_packages(include=["jxORM", "jxORM.orm"]),  # 自动查找包
    package_dir={
        "jxORM": "jxORM",  # 指定包根目录
    },
    exclude_package_data={
        "": [".idea/", "__pycache__/"],
        "jxORM": ["__pycache__/"],
    },
    install_requires=[  # 依赖项
        'DBUtils',
        'pytz',
        'PyMySQL',
    ],
    long_description='''

jxORM是配套jxWebUI使用的数据库操作库。

### 使用说明

jxORM的使用非常简单，主要包括几个步骤：

1、导入依赖

	from jxORM import jxORMLogger, ORM, DBDataType, ColType, jxDB

2、设置数据库连接
    
    #用默认设置，设置本地的mysql数据库连接
	jxDB.set('testDB', password='password')

目前，jxORM支持mysql和sqlite两种数据库。

3、定义一个类，然后用ORM进行修饰，以表明其是一个jxORM数据类

    @ORM
	class User:
        ID:DBDataType.Long = ColType.PrimaryKey
        CreateTime:DBDataType.DataTime = 1
        Nameame:DBDataType.Chars = 2
        Type:DBDataType.Chars = 2
        Info:DBDataType.Json

上述代码定义了一个数据类也是同名的数据表【User】，其属性如下：

| 名称         | 类型       | 说明          |
|------------|----------|-------------|
| ID         | Long     | 主键          |
| CreateTime | DataTime | 索引1         |
| Name       | 字符串      | 联合索引2       |
| Type       | 字符串      | 联合索引2       |
| Info       | json     | json格式的附属数据 |

4、在数据库中创建数据表

在操作数据前需要先获得数据库连接对象：db。在jxWebUI中，每个事件响应函数在调用时会自动获取到db，我们可以直接使用：

	User.create(db)

这条语句会在数据库中创建一个名为【user】的表，其建表语句如下：

    CREATE TABLE `User` (
    `ID` bigint NOT NULL,
    `CreateTime` datetime NOT NULL,
    `Name` varchar(126) NOT NULL,
    `Type` varchar(126) NOT NULL,
    `Info` mediumtext NOT NULL,
    PRIMARY KEY (`ID`),
    KEY `User_index_1` (`CreateTime`),
    KEY `User_index_2` (`Name`,`Type`)
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci

如果数据库中已经存在名为【User】的表，那么会在日志中记录表已经存在，但不会影响后继的执行。

5、插入数据

    user = User()
    user.Name = 'admin'
    user.Type = 'admin'
    user.Info = {'name': 'admin', 'pwd': '123456'}
    user.insert(db)

则会在表User中插入一条记录。ID、CreateTime会自动生成。

6、修改数据

    user.Name = 'test'
    user.Type = 'normal'
    user.update(db)

7、查询数据

    u = User.Get(db, 'Name == "admin"‘)

如果只执行了5而未执行6，则返回前面创建的admin用户。如果已经执行了6，则返回None。

查询数据有多种方式以适应不同的查询需求，具体请参考【查询数据】。

<font color=red size=3>注：</font>jxORM不支持删除，一般通过设置一个bool型属性来实现逻辑删除

### 扩展数据

经常会遇到新需求需要扩展原有的数据表。jxORM提供了非常简便的扩展数据的方法：继承性扩展：
    
    @ORM
    class User2(User):
        ID:DBDataType.Long = ColType.PrimaryKey
        State:DBDataType.Chars = 1
        Noused:DBDataType.Bool
    User2.create(db)

这就创建了一个新的数据表和数据类：User2。对于数据类User2来说，其具有User类的所有属性，同时还具有新的属性：State和Noused。

    u = User2()
    u.Name = 'test2'
    u.Type = 'xxxxxxxxx'
    u.State = 'ok'
    u.Info = {'a':1, 'b':2}
    u.insert(db)

而User2数据表的建表语句如下：

    CREATE TABLE `User2` (
      `ID` bigint NOT NULL,
      `State` varchar(126) NOT NULL,
      `Noused` tinyint NOT NULL,
      PRIMARY KEY (`ID`),
      KEY `User2_index_1` (`State`)
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci

其只有User2类的定义。所以，上述u.insert(db)是将数据分别插入到User和User2表中。jxORM会自动完成相应的处理。

同理，update和相关的**类数据查询方法**也会自动完成具有继承关系的数据类所对应的数据表之间的关联。

<font color=red size=3>注：</font>具有继承关系的数据表必须定义Long型的ID属性为主键，而且所有子类也都必须如此，否则jxORM无法完成关联。

这种方式的最大好处是在扩展数据后，原本使用User的老代码，不受任何影响【如，原本修改User表新增字段的方式，如果新增字段定义为NOT NULL，老代码插入数据时就会因无法插入而崩溃】。新的代码则使用User2即可。

使用了User2的新代码，在查询时由于jxORM会自动关联User和User2表，而老代码只使用了User表，所以老代码插入的数据，新代码无法查询的到。同理，其它通过继承创建了User3、User4的代码也会和User2形成隔离。

### 日志

引用jxORM后，会在当前目录下的logs子目录【没有则会自动创建】中会创建一个日志文件：

- jxORM.log：是jxORM的运行日志，会记录所有的数据库操作等

日志都是30个日志文件、每个日志文件500M进行循环，所以如长期运行需注意硬盘空间的使用情况。

### 安装jxORM

	pip install jxORM

### 接入jxWebUI

按下述代码即可将jxORM接入到jxWebUI中：

    from jxWebUI import jxWebSQLGetDBConnection
    from jxORM import get_default_db
    
    jxWebSQLGetDBConnection(get_default_db, is_custom=True)

    ''',
    long_description_content_type="text/markdown",
    author='徐晓轶',
    author_email='andrew@pythonpi.top',
    description='jxORM是配套jxWebUI使用的数据库操作库',
    url='https://blog.csdn.net/jxandrew/article/details/14946353',  # 项目的URL
    python_requires=">=3.8",
)
